;; -----------------------------------------------------------------------
;;   
;;   Copyright 1994-2003 H. Peter Anvin - All Rights Reserved
;;   Copyright 2004 Rmi Lefvre - All Rights Reserved
;;
;;   This program is free software; you can redistribute it and/or modify
;;   it under the terms of the GNU General Public License as published by
;;   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
;;   Bostom MA 02111-1307, USA; either version 2 of the License, or
;;   (at your option) any later version; incorporated herein by reference.
;;
;; -----------------------------------------------------------------------

;;
;; sdi.inc
;;
;; Common code for running a SDI image
;;

%define SDI_PHY_ADDR	1000000h	; High memory physical address where
					; SDI image will be copied, 
					; should be 4K page-aligned and above
					; 4MB

is_sdi_image:
	push si			; Save file handle

	mov si,loading_sdi_msg
	call cwritestr

	movzx eax,ax		; Put Kernel size in eax
	shl edx,16		;
	or eax,edx		;
	mov [KernelSize],eax	; Save size
%if IS_ISOLINUX || IS_PXELINUX
	mov si,sdi_size_msg
	call cwritestr
	call writehex8		; Print size
%endif	

	xor edx,edx
	div dword [ClustSize]	; # of clusters total
	add edx,byte -1		; round up
	adc eax,byte 0
	mov [KernelClust],eax	; save it

;;
;; Transfer first half/moby
;;

	push word real_mode_seg
	pop es
	call abort_check
	mov ecx,[ClustPerMoby]
	shr ecx,1		; Half a moby
	cmp ecx,[KernelClust]
	jna .normalsdi
	mov ecx,[KernelClust]	; Kernel smaller than half a moby
.normalsdi:
	sub [KernelClust],ecx
	xor bx,bx
	pop si
	call getfssec
	cmp dword [es:0], '$SDI'
	jne sdi_corrupt		; SDI signature missing

	push si			; Save cluster pointer for later
	mov si,sdi_sig_ver	; SDI sig found
	call cwritestr
	mov al, [es:4]		;
	call writechr		;  
	mov al, [es:5]		; Print version
	call writechr		;  
	mov al, [es:6]		;
	call writechr		;  
	mov al, [es:7]		;
	call writechr
	mov eax, dword [es:16]
	cmp eax,0
	jz sdi_no_bootblob
	mov [BootCodeOffLow], eax
	mov eax, dword [es:24]
	cmp eax,0
	jz sdi_null_bootblob
	mov [BootCodeSizeLow], eax
%if IS_ISOLINUX || IS_PXELINUX
	mov si,sdi_code_off_msg		; Print Boot Code offset
	call cwritestr
	mov eax,[BootCodeOffLow]
	call writehex8
	mov si,sdi_code_size_msg	; Print Boot Code size
	call cwritestr
	mov eax,[BootCodeSizeLow]
	call writehex8
%endif	
	
	call highmemsize

	mov si,crlfloading_msg
	call cwritestr
	mov si,KernelCName		; Print SDI image name
	call cwritestr
	mov si,dotdot_msg
	call cwritestr

	mov eax,[HighMemSize]		; Check there is enough high mem to
	sub eax,SDI_PHY_ADDR		; the sdi image
	cmp eax,[KernelSize]
	jb no_high_mem

	xor esi,esi			; Copy from start of SDI image
	add esi, (real_mode_seg << 4)	; Pointer to source in ESI
	mov edi,SDI_PHY_ADDR
	mov ecx,8000h			; 32K

	call bcopy			; 32bits copy to highmem

;;
;; Transfer the remainder in highmem (at EDI)
;;

	mov si,dot_msg			; Progress report
	call cwritestr
	call abort_check

	pop si				; Restore file handle/cluster pointer
	mov eax,[KernelSize]
	sub eax,8000h
	jbe sdi_loaded

	call load_high

sdi_loaded:

;;
;; Copy boot code and... boot
;;
	mov esi,SDI_PHY_ADDR
	mov eax,[BootCodeOffLow]
	add esi,eax

	mov edi,7C00h
	mov ecx,[BootCodeSizeLow]

	mov edx,SDI_PHY_ADDR		; EDX = physical address of SDI 
	or dl, 41h			; bitwise ORed with 041h

	pushfd
	pushad
	push ds
	push es
	push fs
	push gs

	jmp bcopy_over_self

sdi_corrupt:
	mov si, err_notsdi
	jmp abort_load
sdi_null_bootblob:
	mov si, err_null_bootblob
	jmp abort_load
sdi_no_bootblob:
	mov si, err_no_bootblob
	jmp abort_load

;;
;; Should be in DATA Section
;;
err_notsdi		db CR, LF, 'Invalid or corrupt SDI image.', CR, LF, 0
err_no_bootblob		db CR, LF, 'No BOOT BLOB found.', CR, LF, 0
err_null_bootblob	db CR, LF, 'BOOT BLOB has a null size.', CR, LF, 0
loading_sdi_msg		db CR, LF, 'SDI image', 0
sdi_sig_ver		db CR, LF, 'Signature version: ', 0
%if IS_ISOLINUX || IS_PXELINUX
sdi_size_msg		db CR, LF, 'Size (bytes): 0x', 0
sdi_code_off_msg	db CR, LF, 'Boot Code Offset 0x', 0
sdi_code_size_msg	db CR, LF, 'Boot Code Size   0x', 0
%endif
BootCodeOffLow		dd 0
BootCodeSizeLow		dd 0
